import Toybox.Graphics;
import Toybox.Lang;
import Toybox.System;
import Toybox.WatchUi;

//this file contains the code for settings - both app-settings (phone or Garmin Express)
//as well as setting for the app on the dive itself

class ComplexWFSettings {
    var bgColor=Graphics.COLOR_BLACK;
    var timeColor=Graphics.COLOR_BLUE;
    var messageColor=Graphics.COLOR_ORANGE;
    var hrColor=Graphics.COLOR_PINK;
    var leadZero=false;
    var hasProperties=false;

    function initialize() {
        hasProperties=(Toybox.Application has :Properties);
        load();         //app settings
        loadLocal();    //on-device settings
    }

    //load app settings.  From Application.Properties
    function load() {
        bgColor=getColor("bgcolor",bgColor);
        timeColor=getColor("timecolor",timeColor);
        messageColor=getColor("messagecolor",messageColor);
        hrColor=getColor("hrcolor",hrColor);
    }

    // load local settings. from Application.Storage
    function loadLocal() {
        leadZero=Application.Storage.getValue("leadzero");
        if(leadZero==null) {leadZero=false;}
    }

    //save changes to on device setting
    function saveLocal() {
        Application.Storage.setValue("leadzero",leadZero);
    }

    hidden function getColor(key,thisDefault) as Number {
	    var colors=[Graphics.COLOR_WHITE,Graphics.COLOR_LT_GRAY,Graphics.COLOR_DK_GRAY,Graphics.COLOR_BLACK,	//0-3
		    Graphics.COLOR_RED,Graphics.COLOR_DK_RED,Graphics.COLOR_ORANGE,Graphics.COLOR_YELLOW,				//4-7
		    Graphics.COLOR_GREEN,Graphics.COLOR_DK_GREEN,Graphics.COLOR_BLUE,Graphics.COLOR_DK_BLUE,			//8-11
		    Graphics.COLOR_PURPLE,Graphics.COLOR_PINK,Graphics.COLOR_TRANSPARENT];

    	var v=readKeyInt(key,-1);
    	if(v<0 || v>colors.size()) { return thisDefault; }
   		return colors[v];
   	}
   	
	hidden function readKeyInt(key,thisDefault) {	
	    var value=null;
        try {
            value = Application.Properties.getValue(key);
        } catch(e) {
            value=null;
        }
	    if(value==null || !(value instanceof Number)) {
			if(value!=null) {value=value.toNumber();} 
			else {value=thisDefault;}
    	}    	
		return value;    
	}
}

class ComplexWFSettingsMenu extends WatchUi.Menu2 {
    var mySettings=null;

    function initialize() {
        Menu2.initialize(null);
        mySettings=new ComplexWFSettings();
        Menu2.setTitle("Settings");

 		Menu2.addItem(new WatchUi.ToggleMenuItem("Leading Zero for Hours", null,"lz",mySettings.leadZero, null));
    }
}

class ComplexWFSettingsMenuDelegate extends WatchUi.Menu2InputDelegate {
	var view=null;

    function initialize(v) {
        Menu2InputDelegate.initialize();
        view=v;
    }
    
  	function onSelect(item) {
  		var id=item.getId();
  		if(id.equals("lz"))   {view.mySettings.leadZero=!view.mySettings.leadZero;}   	
	}
    
    function onBack() {
        view.mySettings.saveLocal();
        WatchUi.popView(WatchUi.SLIDE_IMMEDIATE);
		//return false;
    }

}	
