import Toybox.Graphics;
import Toybox.Lang;
import Toybox.System;
import Toybox.WatchUi;
import Toybox.Complications;

class ComplexWFView extends WatchUi.WatchFace {
    var width,height,centerW,centerH;
    var timeY,timeF=Graphics.FONT_NUMBER_MEDIUM;
    var timeH,secW,secX;
    var biY;
    var smallF=Graphics.FONT_SMALL,smallH;
    var mySettings;
    var hrY;
    var bgY;
    var pushStepY;

    //misc
    var is24Hour=false;

    //low/high power state
    var inLowPower=false;
    
    //onPartialUpdate things
    var canDoOP=false;
    var doOP=false;
    var lastHr=0;

    //vector font
    var hasScalable=false;
    var vectorF=null;
    var vectorH=0;
    var vectorR=0;

    //burnin - AMOLED vs MIP
    var canBurnIn=false;

    //complications
    var hasComplications=false;
    var hrComp=null;
    var hrId=null;
    var curHr=0;

    //wheelchair
    var hasPushes=false;
    var usePushes=false;

    //background
    var bgMessage="??:??";
    var bgMessageKey="bgmessage";

    function initialize() {
        WatchFace.initialize();

        //read settings
        mySettings=new ComplexWFSettings();
		
        //check for onPartialUpdate
        if( Toybox.WatchUi.WatchFace has :onPartialUpdate ) {
			doOP=true;
			canDoOP=true;
        }

        //check if AMOLED
        var sSettings=System.getDeviceSettings();
        if(sSettings has :requiresBurnInProtection) {       
        	canBurnIn=sSettings.requiresBurnInProtection;        	
        }
        
        // get 12/24 mode
        is24Hour=sSettings.is24Hour;

        //check for scalable founts
        hasScalable=(Toybox.Graphics has :getVectorFont); 

        //check for background service
    	if(Toybox.System has :ServiceDelegate) {
    		Background.registerForTemporalEvent(new Time.Duration(5 * 60));
    	} else {
    		bgMessage="Not Avail";
    	}

        //check for complications
        hasComplications=(Toybox has :Complications); 
        if(hasComplications) {
            hrId=new Id(Complications.COMPLICATION_TYPE_HEART_RATE);
            if(hrId!=null) {
                hrComp = Complications.getComplication(hrId);
                Complications.registerComplicationChangeCallback(self.method(:onComplicationChanged));
                Complications.subscribeToUpdates(hrId);        
            }
        }

        //read stored data
        if(mySettings.hasProperties) {
            var t=Application.Storage.getValue(bgMessageKey);
            if(t!=null) {
                bgMessage=t;   
            }
        }

        //wheelchair mode
        var am=ActivityMonitor.getInfo();
		
        hasPushes=(am has :pushes);	
		if(hasPushes) {
			usePushes=(am.stepGoal==0 || am.stepGoal==null || am.pushes!=0);
		}
    }

    function onLayout(dc as Dc) as Void {
        width=dc.getWidth();
        centerW=width/2;
        height=dc.getHeight();
        centerH=height/2;
        biY=height/4;
        timeH=dc.getFontHeight(timeF);
        secW=dc.getTextWidthInPixels(":88", timeF)+5;
        smallH=dc.getFontHeight(smallF);
        if(hasScalable) {
            vectorH=smallH-5;
            vectorF=Graphics.getVectorFont({:face=>["RobotoCondensedRegular","RobotoRegular"], :size=>vectorH});
            vectorR=centerH-smallH;
        }

        timeY=centerH-(timeH/2);
        hrY=timeY+timeH;
        bgY=timeY-smallH;
        pushStepY=hrY+smallH;
    }

    function onShow() as Void {
        mySettings.loadLocal();
    }

    // Update the view
    function onUpdate(dc as Dc) as Void {
        if(canDoOP) {dc.clearClip();}
        var now=Time.now();
        var clockTime = Time.Gregorian.info(now, Time.FORMAT_MEDIUM);
        var hour=clockTime.hour;
        
        if(!is24Hour) {
			hour = hour % 12;
            hour = (hour == 0) ? 12 : hour;
        }	
        
        var hFormat=(mySettings.leadZero) ? "%02d" : "%d";

        var hhmm=hour.format(hFormat)+":"+clockTime.min.format("%02d");
        if(inLowPower && canBurnIn) {
            //always black for burnin protection
            dc.setColor(Graphics.COLOR_BLACK,Graphics.COLOR_BLACK);
            dc.clear();
            dc.setColor(mySettings.timeColor, Graphics.COLOR_TRANSPARENT);
            dc.drawText(centerW,biY,timeF,hhmm,Graphics.TEXT_JUSTIFY_CENTER+Graphics.TEXT_JUSTIFY_VCENTER);
            biY=(biY+centerH)%height;
        } else {
            dc.setColor(mySettings.bgColor, mySettings.bgColor);
            dc.clear();
            dc.setColor(mySettings.timeColor, Graphics.COLOR_TRANSPARENT);
            var offsetTime=centerW;
            if(!inLowPower || doOP) {offsetTime=centerW-(secW/2);}
            dc.drawText(offsetTime,timeY,timeF,hhmm,Graphics.TEXT_JUSTIFY_CENTER);
            secX=offsetTime+(dc.getTextWidthInPixels(hhmm, timeF)/2);
            if(!inLowPower || doOP) {doSecs(dc,false,secX,timeY,clockTime.sec);}

            dc.setColor(mySettings.messageColor, Graphics.COLOR_TRANSPARENT);
            var dateStr=clockTime.day_of_week.substring(0,3)+" "+clockTime.day;
            if(vectorF!=null) {
                dc.drawRadialText(centerW, centerH, vectorF, dateStr, Graphics.TEXT_JUSTIFY_CENTER, 90, vectorR, Graphics.RADIAL_TEXT_DIRECTION_CLOCKWISE);
            } else {
                dc.drawText(centerW,smallH,smallF,dateStr,Graphics.TEXT_JUSTIFY_CENTER);
            }

            dc.drawText(centerW,bgY,smallF,"BackG: "+bgMessage,Graphics.TEXT_JUSTIFY_CENTER);
            
            var am=ActivityMonitor.getInfo();
            var ps=(usePushes) ? "P: "+am.pushes : "S: "+am.steps;
            dc.drawText(centerW,pushStepY,smallF,ps,Graphics.TEXT_JUSTIFY_CENTER);

            dc.setColor(mySettings.hrColor, Graphics.COLOR_TRANSPARENT);
            var hr=(hasComplications and curHr!=null) ? curHr : Activity.getActivityInfo().currentHeartRate;
            if(hr==null || hr==0 || hr==255) {hr="--";}
            dc.drawText(centerW,hrY,smallF,"HR: "+hr,Graphics.TEXT_JUSTIFY_CENTER);
        }
    }

    function onPartialUpdate(dc) {
        var now=Time.now();
        var clockTime = Time.Gregorian.info(now, Time.FORMAT_SHORT);
        doSecs(dc,true,secX,timeY,clockTime.sec);
    }
    function doSecs(dc,partial,x,y,sec) {
        if(partial) {
            dc.setClip(x, y, secW, timeH);
            dc.setColor(mySettings.bgColor,mySettings.bgColor);
            dc.clear();
        }
        dc.setColor(mySettings.timeColor, Graphics.COLOR_TRANSPARENT);
        dc.drawText(x,y,timeF,":"+sec.format("%02d"),Graphics.TEXT_JUSTIFY_LEFT);
    }

    function onComplicationChanged(id as Complications.Id) as Void {
        if(id.equals(hrId)) {
            curHr=Complications.getComplication(id).value;
        }

    }

    function onHide() as Void {
    }

    // The user has just looked at their watch. Timers and animations may be started here.
    function onExitSleep() as Void {
        inLowPower=false;
        WatchUi.requestUpdate();
    }

    // Terminate any active timers and prepare for slow updates.
    function onEnterSleep() as Void {
        inLowPower=true;
        WatchUi.requestUpdate();
    }

}

class ComplexWFDeligate extends WatchUi.WatchFaceDelegate
{
	var view;
	
	function initialize(v) {
		WatchFaceDelegate.initialize();
		view=v;	
	}

    function onPress(evt) {
        var c=evt.getCoordinates();
        if(c[1]>=view.hrY  && c[1]<view.pushStepY && view.hrId!=null) {
            Complications.exitTo(view.hrId);
        }
        return true;
    }
	
    // onPartialUpdate is no longer called
    function onPowerBudgetExceeded(powerInfo) {
        view.doOP=false;
        WatchUi.requestUpdate();
    }
}
